# A dynamic priority queue. Implements standard +push+, +pop+, and
# +peek+ operations and allows efficient merging of two +SkewHeap+s
#
# Author:: Paul J Sanchez (mailto:pjs@alum.mit.edu)
# Copyright:: Copyright (c) Paul J Sanchez
# License:: LGPL
#
class SkewHeap

  attr_reader :root
  protected :root

  # Construct and initialize a new +SkewHeap+.
  # If a comparator block is provided it will be used to determine ordering
  # for the data, otherwise the default ordering will be used.
  #
  # *Arguments*::
  #   - +ary+ -> an optional array of data to add to the new SkewHeap.
  #
  def initialize(ary=[], &block)
    @cmp_fn = block || lambda {|x,y| x <=> y}
    @root = nil
    ary.each {|item| push(item)} unless ary.empty?
  end

  # Add the provided item to the SkewHeap.
  def push(item)
    @root = __merge__(@root, Node.new(item)) unless item.nil?
  end

  # Remove and return the first element in the SkewHeap.
  def pop
    if @root
      return_value = @root.payload
      @root = __merge__(@root.left, @root.right)
      return_value
    end
  end

  # View the first element in the SkewHeap without removing it.
  def peek
    @root.payload if @root
  end

  # Test whether or not the SkewHeap is empty.
  def empty?
    @root.nil?
  end

  # Remove all elements from the SkewHeap.
  def clear
    @root = nil
  end

  # Combine two Skewheaps.
  #
  # *Arguments*::
  #   - +other+ -> the other SkewHeap to merge into this one.
  # *Returns*::
  #   - a reference to the combined SkewHeap.
  # *Raises*::
  #   - RuntimeError if +other+ is not a SkewHeap
  #
  def merge(other)
    if other.kind_of? SkewHeap
      @root = __merge__(@root, other.root)
      self
    else
      raise "Attempt to merge a non-SkewHeap with a SkewHeap."
    end
  end

  alias join merge

  private

  # Classic tree node with a data payload and left and right references.
  Node = Struct.new(:payload, :left, :right)

  def __merge__(h1, h2)
    return h2 if h1.nil?
    return h1 if h2.nil?
    if @cmp_fn[h1.payload, h2.payload] < 1
      h1.right, h1.left = h1.left, __merge__(h1.right, h2)
      return h1
    else
      h2.right, h2.left = h2.left, __merge__(h2.right, h1)
      return h2
    end
  end

end
